@@ -13,7 +13,6 @@ const MainWindow = imports.mainWindow;
 const PasteManager = imports.pasteManager;
 const Utils = imports.utils;
 
-
 const MAX_RETRIES = 3;
 
 const ConnectionError = {
@@ -41,6 +40,7 @@ const Application = new Lang.Class({
 
         this._chatroomManager = ChatroomManager.getDefault();
         this._accountsMonitor = AccountsMonitor.getDefault();
+        this._networkMonitor = Gio.NetworkMonitor.get_default();
 
         this._accountsMonitor.connect('account-removed', Lang.bind(this,
             function(am, account) {
@@ -70,6 +70,9 @@ const Application = new Lang.Class({
           { name: 'message-user',
             activate: Lang.bind(this, this._onMessageUser),
             parameter_type: GLib.VariantType.new('(ssu)') },
+          { name: 'paste-text',
+            activate: Lang.bind(this, this._onPasteText),
+            parameter_type: GLib.VariantType.new('s') },
           { name: 'leave-room',
             activate: Lang.bind(this, this._onLeaveRoom),
             parameter_type: GLib.VariantType.new('(ss)') },
@@ -77,11 +80,16 @@ const Application = new Lang.Class({
             activate: Lang.bind(this, this._onLeaveCurrentRoom),
             create_hook: Lang.bind(this, this._leaveRoomCreateHook),
             accels: ['<Primary>w'] },
+          { name: 'reconnect-account',
+            parameter_type: GLib.VariantType.new('o') },
           { name: 'user-list',
             activate: Lang.bind(this, this._onToggleAction),
             create_hook: Lang.bind(this, this._userListCreateHook),
             state: GLib.Variant.new('b', false),
             accels: ['F9', '<Primary>u'] },
+          { name: 'edit-connection',
+            activate: Lang.bind(this, this._onEditConnection),
+            parameter_type: GLib.VariantType.new('o') },
           { name: 'connections',
             activate: Lang.bind(this, this._onListConnections) },
           { name: 'preferences',
@@ -255,6 +263,9 @@ const Application = new Lang.Class({
             return;
         }
 
+        if (!this._networkMonitor.network_available)
+            return;
+
         let roomId = Polari.create_room_id(account,  targetId, targetType);
 
         let requestData = {
@@ -302,11 +313,13 @@ const Application = new Lang.Class({
 
         try {
             req.ensure_channel_finish(res);
-
-            if (requestData.targetHandleType == Tp.HandleType.ROOM)
-                this._addSavedChannel(account, requestData.targetId);
         } catch (e if e.matches(Tp.Error, Tp.Error.DISCONNECTED)) {
             let error = account.connection_error;
+            // If we receive a disconnect error and the network is unavailable,
+            // then the error is not specific to polari and polari will
+            // just be in offline state.
+            if (!this._networkMonitor.network_available)
+                return;
             if (error == ConnectionError.ALREADY_CONNECTED &&
                 requestData.retry++ < MAX_RETRIES) {
                     this._retryRequest(requestData);
@@ -330,6 +343,10 @@ const Application = new Lang.Class({
         let [accountPath, channelName, time] = parameter.deep_unpack();
         this._requestChannel(accountPath, Tp.HandleType.ROOM,
                              channelName, time);
+
+        let factory = Tp.AccountManager.dup().get_factory();
+        let account = factory.ensure_account(accountPath, []);
+        this._addSavedChannel(account, channelName);
     },
 
     _onMessageUser: function(action, parameter) {
@@ -338,6 +355,11 @@ const Application = new Lang.Class({
                              contactName, time);
     },
 
+    _onPasteText: function(action, parameter) {
+        let text = parameter.deep_unpack();
+        this.pasteManager.pasteText(text);
+    },
+
     _onLeaveRoom: function(action, parameter) {
         let [roomId, message] = parameter.deep_unpack();
         let reason = Tp.ChannelGroupChangeReason.NONE;
@@ -391,6 +413,19 @@ const Application = new Lang.Class({
             }));
     },
 
+    _onEditConnection: function(action, parameter) {
+        let accountPath = parameter.deep_unpack();
+        let factory = Tp.AccountManager.dup().get_factory();
+        let account = factory.ensure_account(accountPath, []);
+        let dialog = new Connections.ConnectionDetailsDialog(account);
+        dialog.widget.transient_for = this._window.window;
+        dialog.widget.connect('response', Lang.bind(this,
+            function(w, response) {
+                w.destroy();
+            }));
+        dialog.widget.show();
+    },
+
     _onShowPreferences: function() {
     },
 
